
from store_config.service import StoreConfigurationService


class UpdateStoreConfigInteractor:

    def __init__(
        self,
        store_config_service: StoreConfigurationService,
    ) -> None:

        self.store_config_service = store_config_service

    def execute(
        self,
    ) -> bool:

        return self.store_config_service.update_configuration()
