from typing import (
    Dict,
    Any,
)

from store_config.repository import StoreConfigurationRepository


class StoreConfigurationService:

    def __init__(
        self,
        store_config_repository: StoreConfigurationRepository,
    ) -> None:

        self.store_config_repository = store_config_repository

    def get_configuration(
        self,
    ) -> Dict[str, Any]:

        return self.store_config_repository.get_current_configuration()

    def update_configuration(self) -> bool:

        return self.store_config_repository.update_store_configuration()
