from pathlib import Path

pos_dir = Path(__file__).parents[5]
storecfg_bundles_path = pos_dir / "data" / "server" / "bundles" / "storecfg"
store_configs_path = storecfg_bundles_path / "config.json"
jsons_dir = storecfg_bundles_path / "jsons"
