from typing import (
    Any,
    Dict,
)

from store_config.service import StoreConfigurationService


class GetStoreConfigInteractor:

    def __init__(
        self,
        store_config_service: StoreConfigurationService,
    ) -> None:

        self.store_config_service = store_config_service

    def execute(
        self,
    ) -> Dict[str, Any]:

        return self.store_config_service.get_configuration()
